﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Selection;

namespace HeuristicLab.Algorithms.ALPS {

  [Item("EldersSelector", "Select all individuals which are to old for their current layer.")]
  [StorableClass]
  public sealed class EldersSelector : SingleObjectiveSelector {

    private ILookupParameter<IntArray> AgeLimitsParameter {
      get { return (ILookupParameter<IntArray>)Parameters["AgeLimits"]; }
    }
    private ILookupParameter<IntValue> NumberOfLayersParameter {
      get { return (ILookupParameter<IntValue>)Parameters["NumberOfLayers"]; }
    }
    private ILookupParameter<IntValue> LayerParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Layer"]; }
    }

    [StorableConstructor]
    private EldersSelector(bool deserializing)
      : base(deserializing) { }
    private EldersSelector(EldersSelector original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EldersSelector(this, cloner);
    }
    public EldersSelector()
      : base() {
      Parameters.Add(new LookupParameter<IntArray>("AgeLimits"));
      Parameters.Add(new LookupParameter<IntValue>("NumberOfLayers"));
      Parameters.Add(new LookupParameter<IntValue>("Layer"));
    }

    protected override IScope[] Select(List<IScope> scopes) {
      var ageLimits = AgeLimitsParameter.ActualValue;
      int numberOfLayers = NumberOfLayersParameter.ActualValue.Value;
      int layer = LayerParameter.ActualValue.Value;

      if (layer >= numberOfLayers) // is max layer?
        return new IScope[0];

      int limit = ageLimits[layer];
      var elders =
        from scope in scopes
        let age = ((IntValue)scope.Variables["Age"].Value).Value
        where age > limit
        select scope;

      return elders.ToArray();
    }
  }
}